;*******************************************************
;ZMR - Zoomin' Mob Routine (Grayscale Version)
;The Clipped PutSprite by Dux Gregis
;Assembly Coder's Zenith	1999
;Modified by Ricky Cobb <arcadesdude@myrealbox.com.net> 2003
;
;Putsprite now does 8x8 clipping and optional masking
;w/o background saving and grayscale
;
; All routines do clipping (except if sprite's aligned)
;
; 	PutSpriteClip  -- black/white with clipping	  (sprite, mask)
;  PSClipNoMask   -- black/white with clipping and no masking (sprite)
; 	GraySpriteClip -- grayscale with clipping	  (dark, light, mask)
;  GSClipNoMask   -- grayscale with clipping and no masking (dark, light)
;*******************************************************
Plane1 = Buffer3
Plane2 = Buffer4
Plane3 = Buffer1
Plane4 = Buffer2
;*******************************************************
;Input for all routines: hl points to sprite
;                        b,c = x,y
;Need 1 byte temp var called masktemp
;*******************************************************
;Must set masking on or off before you use routine
;
;Maskingoff:
; sub a
; ld (masktemp),a ;masking off
;For example you would turn masking off when drawing
;non-masked tile for a tilemap (background)
;Maskingon:
; ld a,1
; ld (masktemp),a
;Masking on is good for forground sprites/tiles/layers where
;the background would "peek through" where the mask is 0
;*******************************************************
;
;the following addresses contain the clip lines (must be a multiple of 8):

;for PutSprite
;x@setBottomClip	default is 56
;x@setRightClip		default is 120

;this is so you can adjust the position of your status bar
;the sprite will clip left and top at 0

;Zoomin' Mob Routine (PutSprite)
;Input: bc = x,y; hl = 8x8 sprite location
;Output: clipped sprite; all registers destroyed

;the sprite is 8 bytes, followed by an 8 byte mask
;the background storage buffer is 8 bytes (no background storage buffer in this modified routine)

;PutSpriteClip: ;monochrome with clipping and masking
; ld a,1
; ld (masktemp),a ;masking on
; jp putsprite
;PSClipNoMask: ;monochrome with clipping
; sub a
; ld (masktemp),a ;masking off
; jp putsprite

GraySpriteClip: ;grayscale with clipping and masking, hl=sprite, bc=x,y
 ex de,hl
; ld a,1
; ld (masktemp),a ;masking on
 ld hl,Plane3
 ld (ps@addvid),hl
 ld a,16
 ld (psLoop@mask),a
 ex de,hl
 push hl
 push bc
 call putsprite
 pop bc
 ld hl,Plane4
 ld (ps@addvid),hl
 ld a,8
 ld (psLoop@mask),a
 pop hl
 ld de,8
 add hl,de
 jp putsprite

GSClipNoMask: ;grayscale with clipping
 ex de,hl
; sub a
; ld (masktemp),a ;masking off
 ld hl,Plane1
 ld (ps@addvid),hl
 ex de,hl
 push hl
 push bc
 call putsprite
 pop bc
 ld hl,Plane2
 ld (ps@addvid),hl
 pop hl
 ld de,8
 add hl,de
; jp putsprite

PutSprite:
 push hl
 pop ix
 ld d,0 ;set leftdown fix to zero (will add to x@clipVert in ClipDown) part of overwrite fix
 ld a,b
 or a								;test x coordinate
 jp m,clipLeft						;clip left if negative
x@setRightClip	equ		$+1
 sub 120							;if (x coordinate - x clip line >= 0)
 jp p,clipRight						;then clipRight
retClipHorz:						;point of return for both horizontal clips
 ld a,c
 or a								;test the y coordinate
 jp m,clipUp						;if negative, clip top of sprite
x@setBottomClip	equ		$+1
 sub 56								;if (y coordinate - y clip line > 0)
;next two instructions were first jp z,ps then jp p, clipdown
;changing to the current way fixes the overwrite problem
 jp p,clipDown						;then clipDown
 jp z,ps
ps:									;point of return for both vertical clips
 call findPixel						;pixel coordinates -> video address
ps@addvid = $+1
 ld de,$FC00 ;self modifiying (for optional grayscale)
 add hl,de
 ld (x@bitOffset),a					;store findPixel bit offset
x@UpLeft		equ		$
 nop
x@clipVert		equ		$+1
 ld b,8								;loop 8 times unless vertical clip
 ld c,0
psLoop:
 push bc							;save loop counter
 push hl							;save video mem
 ld b,(ix)							;sprite row -> BC
 ld a,(masktemp)
 or a ;check optional masking [0,z no masking] [1,nz masking]
 ld a,$ff ;no masking byte (unless masking is set then it gets it)
 jr z,psskipmasking ;skip next instruction
 ld a,(ix+8)						;mask row -> AD ;self modifying
psLoop@mask = $-1
psskipmasking:
 ld d,c
x@bitOffset		equ		$+1
 ld e,0								;bit offset -> E
 dec e
 jp m,noShift
bitShift:							;shift BC, AD right; HL left E times
 srl b								;sprite
 rr c
 rra								;mask
 rr d
 dec e
 jp p,bitShift
 inc e								;0 -> E
x@clipHorz		equ		$
 nop								;clear B,A if clip left; C,D if clip right
 nop								;(E is 0, so load into unwanted registers)
noShift:
 pop hl ;get vid mem
 or a ;if a=0 that means x@clipHorz was set to clear a so dont draw left byte
 jp z,noShiftskipleftbyte ;jp is faster
 cpl								;invert the sprite mask
 and (hl)							;AND it with video memory
 or b								;OR it with sprite row
 ld (hl),a							;load it to video memory
noShiftskipleftbyte:
 inc hl
 ld a,d
 or a ;if a=0 that means dont draw right byte
 jp z,noShiftskiprightbyte ;jp is faster
 cpl
 and (hl)
 or c
 ld (hl),a
noShiftskiprightbyte:
 inc ix								;increment sprite row pointer
 ld bc,15
 add hl,bc							;point to next video row
 pop bc								;loop counter -> B
 djnz psLoop
doneps:
 ld h,b								;clear HL
 ld l,b
 ld (x@clipHorz),hl					;fix horizontal clipping
 xor a
 ld (x@UpLeft),a
 ld a,8
 ld (x@clipVert),a					;fix vertical clipping
 ret
clipUp:
 add a,8							;if (y <= -8)
 ret m								;then return (display nothing)
 ret z
 ld c,0								;new y coordinate is 0
 ld (x@clipVert),a					;new number of sprite rows
 xor %111
 ccf
 adc a,ixl							;add the inverse to IX (sprite row pointer)
 ld ixl,a
 jp nc,ps
 inc ixh
 jp ps
clipDown:
 sub 8								;if (y <= 64)
 ret p								;then return (display nothing)
 neg
 add a,d ; add to number of rows either 0 or -1 for overwrite fix
 ld (x@clipVert),a					;new number of sprite rows
 jp ps
clipLeft:
 cp -8								;if (x <= -7)
 ret c								;then return (display nothing)
 ret z
 res 7,a							;kill the sign
 ld b,a								;makes for good x coordinate
 dec c								;but for one row down ;(up?)
 ld hl,$437b						;<ld b,e>, <ld a,e> (E is 0)
 ld (x@clipHorz),hl					;move opcodes to clipping area
 ld d,-1                ;add to new number of sprite rows
 jp p,retClipHorz ;if below top line of screen (if not upleft, just left continue)
 inc c
 and 7
 ld b,a
 ld a,$2b							;dec hl ;beastly hack ..
 ld (x@UpLeft),a
 jp retClipHorz
clipRight:
 sub 8
 ret p
 ld hl,$4b53						;<ld c,e>, <ld d,e> (E is 0)
 ld (x@clipHorz),hl					;move opcodes to clipping area
 jp retClipHorz

;---------------------------------------------------------------------
;  An Edited, edited EBLE-YOPP Routine :)
;
;  127 cycles - 35 bytes
;  (b,c) to hl:a - destroyes: de
;---------------------------------------------------------------------
findpixel:
Find_Pixel:
 ld de,FP_Bits
 ld a,b
 and %00000111
 add a,e
 ld e,a
 adc a,d
 sub e
 ld d,a                ;(de)=bitmask
 ld hl,FP_RLD
 ld (hl),c             ;FP_RLD=y
 ld a,b                ;a=x
 rra
 rra
 rra                   ;  x/8
 rld                   ;a(hl)=16y+x/8
 and %00000011
 ld l,(hl)
 ld h,a
 ld a,(de)
 ret
FP_Bits:
.db 0,1,2,3,4,5,6,7 ;and obvious optimization here?

FP_RLD = x@bitOffset ;cause it works good like this :)